/*
 * Модуль памяти.
 * Нужен для сохранения и загрузки информации. Сразу в localStorage писать нельзя, т.к. он уничтожается при закрытии webView.
 * Проверяет параметр config.memory.prefix. Если параметр задан, добавляет префикс для всех ключей.
 * Префикс по умолчанию: Scoring_SDK__
 * 
 * API модуля:
 * ScoringSDK.save - сохранить информацию. Ожидает два обязательных аргумента: ключ и данные.
 * ScoringSDK.load - загрузит информацию. Ожидает один обязательный аргумент: ключ.
 *
 * В оба метода также можно передать дополнительный, необязательный аргумент - приватный префикс игры (как правило это ключ выдаваемый на сборку).
 * Приватный префикс нужен для ограничения видимости параметров конкретно взятой игры.
 * Например поле Ник - общее для всех игр, а Судоку + Префикс - уникальные настройки игры Судоку.
 * 
 * Пример использования модуля:
 * var key = "n4vI27K89v";
 * ScoringSDK.save("nik", "Player");     // Общее поле.
 * ScoringSDK.save("nik", "Pavel", key); // Приватное поле конкретно этой игры.
 * 
 * ScoringSDK.load("nik");      // Вернет общее Player для всех игр.
 * ScoringSDK.load("nik", key); // Вернет приватное Pavel, конкретно этой игры.
 * 
 */
(function (global) {
    "use strict";

    global = global || {};
    global.ScoringSDK = global.ScoringSDK || {};
    var config = global.ScoringSDK._config;

    var module = {
        _prefix: config.memory.prefix || "Scoring_SDK__",
        _storage: {
            save: {
                localStorage: "setItem",
                API: "save"
            },
            load: {
                localStorage: "getItem",
                API: "load"
            }
        },
        _setItem: function (key, value) {
            var storage = this._storage.save;
            for (var method in storage) {
                if (global[method] && global[method][(storage[method])]) global[method][(storage[method])](key, value);
            }
        },
        _getItem: function (key, value) {
            var storage = this._storage.load,
                value = null;
            for (var method in storage) {
                if (global[method] && global[method][(storage[method])]) value = global[method][(storage[method])](key);
            }
            return this._getObject(value);
        },
        _getObject: function (value) {
            try {
                if (global.JSON && global.JSON.parse) return global.JSON.parse(value);
            } catch (e) {
            }
            return value;
        },
        _getString: function (value) {
			var type = typeof value;
            if (type == "object" && global.JSON && global.JSON.stringify) try {
                value = global.JSON.stringify(value);
            } catch (e) {
            }
            if (type == "string") return value;
            if (value.toString) return value.toString();
            return "";
        },
        load: function (key, prefix) {
            if (typeof key != "string") return false;
            return this._getItem((this._prefix + key + (prefix ? "_" + prefix : "")));
        },
        save: function (key, value, prefix) {
            if (typeof key != "string") return false;
            return this._setItem((this._prefix + key + (prefix ? "_" + prefix : "")), this._getString(value));
        }
    };

    global.ScoringSDK.save = function (key, value, prefix) {
        return module.save(key, value, prefix);
    };

    global.ScoringSDK.load = function (key, prefix) {
        return module.load(key, prefix);
    };
})(this);