/*
 * Модуль рейтинга.
 * Возвращает рейтинг и отправляет очки игрока.
 *
 * API модуля:
 * ScoringSDK.getRating - получить текущий рейтинг игроков. Ожидает callback-функцию, в которую передаст массив игроков.
 * ScoringSDK.send - отправить результаты игрока и оплать. Ожидает один аргумент - объект с параметрами. Пример ниже.
 * ScoringSDK.setSendStatus - функция, в которую API системы передает статус оплаты.
 * 
 * Пример объекта для функции send:
 * {
 *     name: "Ваня",                   // Имя игрока
 *     score: 123,                     // Очки игрока
 *     callback: function (status) {   // Callback-функция, которая будет вызвана после оплаты.
 *         ...                         // Аргумент status будет указывать на статус оплаты (true/false)
 *     }
 * }
 * 
 */
(function (global) {
    "use strict";

    global = global || {};
    global.ScoringSDK = global.ScoringSDK || {};
    var config = global.ScoringSDK._config,
        utils = global.ScoringSDK._utils;

    var module = {
        _callback: null,
        _getValidData: function (data) {
            var validData = [];
			if(typeof data == "object" && !(data instanceof Array)) {
				data = [ data ];
			}
            for (var i = 0, l = data.length; i < l; i++) {
                validData.push({
                    name: data[i]["@name"] || "",
                    position: data[i]["@position"] || 0,
                    score: data[i]["@score"] || 0
                });
            }
            return validData;
        },
        getRating: function (callback) {
            var self = this;
            utils.ajax.get((config.rating.url || ""), function (response) {
                var data = [];
                if (response && response.game && response.game.player) {
                    data = self._getValidData(response.game.player);
                }
                if (callback && typeof callback == "function") callback(data);
            });
        },
        _getParameters: function (data) {
            var parameters = [];
            for (var id in config.purchase) {
                parameters.push((id + "=" + encodeURIComponent(config.purchase[id])));
            }
            parameters.push(("score=" + encodeURIComponent(data.score)));
            parameters.push(("name=" + encodeURIComponent(data.name)));
            return parameters.join("&");
        },
        _getTextSMS: function (data) {
            return [
                (config.purchase.prefix_sms || ""),
                " ",
                (config.purchase.prefix_game || ""),
                (config.purchase.language || ""),
                (config.purchase.chanel || ""),
                " ",
                (data.score || "0"),
                "+S ",
                (data.name || ""),
                " "
            ].join("");
        },
        _checkData: function (data) {
            if (typeof data.name != "string" || typeof data.score != "number") return false;
            data.score = Math.round(Math.abs(data.score));
            data.name = data.name.replace(/[\s]+/im, "_") || "";
            if (data.score < 0 || data.score > 100000000) return false;
            if (data.name.length < 3 || data.name.length > 20) return false;
            return true;
        },
        sendScore: function (data) {
            data = data || {};
            var self = this;
            if (!self._checkData(data)) return self.setSendStatus();
            self._callback = null;
            if (data.callback && typeof data.callback == "function") self._callback = function (status) {
                data.callback(status);
            };
            if (global.API && global.API.sendSms && typeof global.API.sendSms == "function") {
                if (config.isBuyWithSMS) {
                    global.API.sendSms(config.purchase.phone, self._getTextSMS(data));
                } else {
                    global.API.sendSms(self._getParameters(data));
                }
            } else {
                self.setSendStatus();
            }
        },
		setEvent: function (type, message) {
            if (global.API && global.API.setEvent && typeof global.API.setEvent == "function") {
				if(type && message && typeof type == "string" && typeof message == "string") {
					global.API.setEvent(type, message);
				}
			}
        },
        setSendStatus: function (status) {
            var self = this;
            self._callback && self._callback(!!status);
            self._callback = null;
        }
    };

    global.ScoringSDK.getRating = function (callback) {
        return module.getRating(callback);
    };

    global.ScoringSDK.send = function (data) {
        return module.sendScore(data);
    };

    global.ScoringSDK.setEvent = function (type, message) {
        return module.setEvent(type, message);
    };

    global.ScoringSDK.setSendStatus = function (status) {
        return module.setSendStatus(status);
    };
})(this);