﻿/*
 * Модуль для создания API для объектов или модулей.
 * Ожидает на входе объект или функцию. В случае объекта - идет перебор методов и поиск таких,
 * имя которых начинается без префикса "_". Все методы оборачиваются в try/catch конструкцию.
 * Таким образом даже если при выполнении модуль падает, обертка API ловит ошибку и программа продолжает работать.
 *
 * API модуля:
 * get()    - получить объект API.
 * set()    - установить объект API.
 * create() - получить и установить объект API.
 * 
 * Пример использования:
 * var module = {
 *     _a: function() { ... },
 *     b: function() { ... }
 * };
 * 
 * API.create(module, "moduleName");
 * 
 * Создаст обертку для b(), но пропустит _a(). Модуль получит имя "moduleName".
 * На выходе получим конструкцию вида:
 * window.core.module.moduleName = {
 *     methodB: function() {
 *         try {
 *             return module.methodB();
 *         } catch(e) {
 *             return arguments[0] || false;
 *         }
 *     }
 * };
 *
 */
(function (global) {
    var config = {
        core: "module",
        StalinGrad: "module"
    };

    var methods = {
        _getCallback: function (callback, context) {
            return function () {
                try {
                    return callback.apply((context || callback), arguments);
                } catch (e) {
                    return arguments[0] || false;
                }
            };
        },
        _toObject: function (data) {
            var api = {};
            for (var id in data) {
                if (id[0] == "_" || typeof data[id] != "function") continue;
                api[id] = this._getCallback(data[id], data);
            }
            return api;
        },
        createScope: function () {
            for (var name in config) {
                if (!global[name]) {
                    global[name] = {};
                }
                if (!global[name][(config[name])]) {
                    global[name][(config[name])] = {};
                }
            }
        },
        get: function (data) {
            var self = this,
                type = typeof data;
            if (type == "object") {
                return self._toObject(data);
            }
            if (type == "function") {
                return self._getCallback(data);
            }
            return false;
        },
        set: function (api, moduleName) {
            var type = typeof api;
            if (typeof moduleName != "string") return false;
            for (var name in config) {
                if (!global[name] || !global[name][(config[name])]) {
                    continue;
                }
                var object = global[name][(config[name])];
                if (type == "function") {
                    object[moduleName] = api;
                } else if (type == "object") {
                    object[moduleName] = object[moduleName] || {};
                    for (var method in api) {
                        object[moduleName][method] = api[method];
                    }
                }
            }
        },
        create: function (object, name) {
            this.set(this.get(object), name);
        }
    };

    methods.createScope();
    methods.create(methods, "api");
})(this);