﻿/*
 * Модуль для контекстного меню.
 * Создает различные меню, согласно заданным параметрам.
 * При инициализации ожидает данные в виде:
 * {
 *     id: "id_меню",
 *     items: [
 *         {
 *             title: "Текст на кнопке",
 *             link: "ссылка, которая отработает при нажатии",
 *             callback: function() {
 *                 функция, которая отработает при нажатии
 *             }
 *         },
 *         {
 *             title: "Текст на второй кнопке"
 * 			   ...
 *         }
 *     ]
 * }
 * 
 * Вернет объект меню с двумя методами и одним свойством:
 * show - покажет меню
 * hide - скроет меню
 * node - указатель на DOM элемент меню
 * 
 * Пример:
 * var menu = contextMenu(данные_для_создания_меню);
 * menu.show();
 * menu.hide();
 * menu.node.appendChild(DOM_элемент);
 * 
 * Модуль требует наличия в документе следующего HTML кода:
 * <table cellspacing="0" cellpadding="0" class="context_menu__hidden" id="context_menu__table">
 *     <tr class="context_menu__row">
 *         <td class="context_menu__cell" id="context_menu__cell"></td>
 *     </tr>
 * </table>
 * 
 */
(function (global) {
    global = global || window;
    global.StalinGrad = global.StalinGrad || {};
    global.StalinGrad.module = global.StalinGrad.module || {};
    var utils = global.StalinGrad.utils;
    global = global.StalinGrad.module;

    var module = {
        _nodes: {},
        _prefix: "context_menu__",
        _isShow: false,
        _activeItem: null,
        data: {},
        _setNodes: function () {
            var arrayID = [
                "table",
                "cell"
            ];
            for (var i = 0, l = arrayID.length; i < l; i++) {
                this._nodes[(arrayID[i])] = document.getElementById((this._prefix + arrayID[i]));
            }
        },
        _setEvents: function () {
            utils.addEvent(this._nodes.table, "click", function (event) {
                utils.stopEvent(event);
            });
        },
        hide: function (node) {
            this._isShow = false;
            node = (node) ? node : this._nodes.table;
            if (node) node.className = "context_menu__hidden";
        },
        show: function (node) {
            var self = this,
                nodes = self._nodes;
            if (!node) return false;
            if (nodes.table) nodes.table.className = "context_menu__table";
            if (self._activeItem) self.hide(self._activeItem);
            node.className = "";
            self._activeItem = node;
            self._isShow = true;
        },
        createContainer: function (id) {
            var self = this,
                node = document.createElement("DIV"),
                parentNode = self._nodes.cell;
            node.className = "context_menu__hidden";
            if (parentNode) parentNode.appendChild(node);
            return node;
        },
        getCallback: function (item) {
            return function (event) {
                utils.stopEvent(event);
                item.callback && item.callback(item);
                if (item.link) window.location.href = item.link;
            };
        },
        createMenu: function (data) {
            var self = this,
                item = {
                    nodes: {
                        items: [],
                        container: self.createContainer(data.id)
                    }
                };
            for (var i = 0, l = data.items.length; i < l; i++) {
                var node = document.createElement("DIV");
                node.className = "context_menu__item";
                node.innerHTML = data.items[i].title || "";
                data.items[i].node = node;
                item.nodes.items.push(node);
                utils.addEvent(node, "click", self.getCallback(data.items[i]));
                item.nodes.container.appendChild(node);
            }

            self.data[data.id] = item;
            return new menu(item.nodes);
        },
        disable: function(nodes, index) {
            if(!nodes || !nodes[index]) return false;
            nodes[index].className = "context_menu__hidden"
        },
        enable: function(nodes, index) {
            if(!nodes || !nodes[index]) return false;
            nodes[index].className = "context_menu__item"
        },
        init: function () {
            var self = this;
            self._setNodes();
            self._setEvents();
        }
    };


    function menu(nodes) {
        this.nodes = nodes;
        this.isShow = false;
    }

    menu.prototype.show = function () {
        module.show(this.nodes.container);
    };

    menu.prototype.hide = function () {
        module.hide();
    };

    menu.prototype.disable = function (index) {
        module.disable(this.nodes.items, index);
    };

    menu.prototype.enable = function (index) {
        module.enable(this.nodes.items, index);
    };

    menu.prototype.toggle = function () {
        var self = this;
        if (self.isShow && module._isShow) {
            self.isShow = false;
            module.hide();
        } else {
            self.isShow = true;
            module.show(self.nodes.container);
        }
    };

    global.contextMenu = function (data) {
        module.init();
        return module.createMenu(data);
    }
})(window);