/*
 * Модуль, который следит за состоянием хэша страницы. Проверка на изменение - каждые 200 миллисекунд.
 * Ожидает начало хэша с #! т.к. поисковики ищут этот параметр на Ajax сайтах.
 *
 * Структура модуля:
 *   _status        <- Статус хэша. Нужен при создании Android приложений.
 *                     TRUE - хэш изменился из-за события в JavaScript.
 *                     FALSE - хэш изменился, из-за действий полользователя в адресной строке.
 *   _oldHash 		<- Значение старого хэша
 *   _timer 		<- Ссылка на экземпляр таймера, по которому производится проверка хэша.
 *   _intervalChecking  <- Интервал проверки хэша страницы в милисекундах
 *   _defaultData 	<- Данные в хэше по умолчанию
 *   _parameters 	<- Параметры в хэше на текущий момент.
 *   _startChecking <- Функция запускающая проверку хэша по таймеру.
 *   _addStatusInData   <- Добавляет статус хэша в параметры ответа.
 *   _check 		<- Функция проверки хэша. Проверяет изменился хэш или нет.
 *   _getParameters <- Фукнкия парсинга хэша. Возвращает набор параметров.
 *   _setHash 		<- Функция для установления нового хэша страницы, согласно текущим параметрам
 *   addDataInHash 	<- Функция добавления новых данных в хэш
 *   removeDataFromHash <- Функция удаления данных из хэша
 *   _setEvents 	<- Функция установки событий прослушки и публикации
 *   initialization <- Функция первоначальной иницилизации модуля
 *   init 			<- function
 *
 * Слушает события:
 * hash_controller__init - инициализация модуля. Ожидает объект с параметрами, которые будут установленны как параметры
 *                         по умолчанию при запуске страницы.
 * hash_controller__add  - добавляет новый параметр в хэш. Если такой параметр в хэше уже существует - перезатирает
 *                         параметр. Ожидает объект с параметрами. Например:
 * {
 *     page: "main"
 * }
 *
 * hash_controller__remove - Удаление всех параметров из хэша и замену их новыми параметрами.
 *                           Ожидает объект с параметрами. Например:
 * {
 *     page: "main"
 * }
 *
 * Публикует события:
 * hash_controller__update - вернет объект с параметрами из хэша. Событие публикуется при изменении хэша страницы.
 *                           Объект с параметрами также содержит стандартный параметр status, который указывает на
 *                           причину изменения хэша.
 *
 */
(function (global) {
    "use strict";

    global = global || {};
    var utils = global.StalinGrad.utils;

    var module = {
        _status: false,
        _oldHash: "",
        _timer: null,
        _intervalChecking: 200,
        _defaultData: {},
        _parameters: {},
        _startChecking: function () {
            var self = this;
            clearInterval(self._timer);
            self._timer = setInterval(function () {
                self._check();
            }, self._intervalChecking);
        },
        _addStatusInData: function (parameters) {
            var object = {};
            for (var id in parameters) {
                object[id] = parameters[id];
            }
            object.status = this._status;
            this._status = false;
            return object;
        },
        _check: function () {
            var self = this,
                hash = window.location.hash;
            hash = hash.replace(/(\#\!)+/gim, "");
            if (hash == self._oldHash) return false;
            self._oldHash = hash;
            var data = self._getParameters(hash);
            utils.event.trigger("hash_controller__update", self._addStatusInData(data));
        },
        _getParameters: function (hash) {
            var self = this;
            if (hash == "") return self._defaultData;
            hash = hash.split("&");
            for (var i = 0, l = hash.length; i < l; i++) {
                var item = hash[i].split("=");
                if (item.length == 2) self._parameters[item[0]] = item[1];
            }
            return self._parameters;
        },
        _setHash: function () {
            var parameters = this._parameters,
                hash = "#!";
            for (var item in parameters) hash += item + "=" + parameters[item] + "&";
            window.location.hash = hash.replace(/\&$/, "");
        },
        _checkStatusForParameters: function (a, b) {
            if (a != b) this._status = true;
        },
        addDataInHash: function (data) {
            var self = this;
            if (!data || typeof data != "object") return false;
            for (var id in data) {
                self._checkStatusForParameters(self._parameters[id], data[id]);
                self._parameters[id] = data[id];
            }
            self._setHash();
        },
        removeDataFromHash: function (data) {
            if (!data || typeof data != "object") return false;
            var parameters = this._parameters;
            for (var id in data) if (parameters[id]) delete parameters[id];
            this._setHash();
        },
        _setEvents: function () {
            var self = this;
            utils.event.listen("hash_controller__add", function (data) {
                self.addDataInHash(data);
            });
            utils.event.listen("hash_controller__remove", function (data) {
                self._status = true;
                self.removeDataFromHash(data);
            });
            utils.event.listen("hash_controller__init", function (data) {
                self._status = true;
                self.initialization(data);
            });
        },
        initialization: function (data) {
            var self = this;
            self._defaultData = data || {};
            self.addDataInHash(data);
            self._startChecking();
        },
        init: function () {
            this._setEvents();
        }
    };

    module.init();
})(this);