/*
 * Класс "Иннап-сервис"
 * Создает объекты иннап-сервиса для разлочки чего-либо путем отправки СМС. Каждый экземпляр класса запрашивает у сервера
 * информацию для СМС (короткий номер, текст сообщения, стоимость). После чего предлагает пользователю отправить сообщение.
 * После отправки сообщения экземпляр запрашивает у сервера статус покупки. Если покупка была совершенна - перестает
 * показывать блокирующее окно.
 *
 * Так же игроку дается начальное время, во время которого он может ознакомиться с игрой. После истечения этого времени
 * сервис начнет вызывать блокирующее окно.
 *
 * Пример создания экземпляра класса:
 * module.innap({
 *     intervalChecking: {
 *         time: 1 * 1000,
 *         innap: 20 * 1000
 *     }
 * });
 *
 * Каждый экземпляр класса имеет следующие API:
 *
 *
 */
(function (global) {
    "use strict";

    global = global || {};
    global.StalinGrad = global.StalinGrad || {};
    var utils = global.StalinGrad.utils || {},
        module = global.StalinGrad.module || {};

    function Innap(userProperty) {
        var self = this,
            defaultProperty = {
                intervalChecking: {
                    time: 1 * 1000,
                    innap: 20 * 1000
                },
                memoryKey: "innap_reversi_",
                prefix: "innap__"
            },
            privateProperty = {
                _nodes: {},
                _timer: {
                    checkTime: null,
                    checkInnap: null
                },
                _unlock: false,
                _window: null,
                _time: null,
                _innap: null
            };

        module.object.set(self, defaultProperty, userProperty);
        module.object.set(self, privateProperty);
    }

    var methods = {
        checkTime: function () {
            var self = this,
                difference = new Date() - self._time,
                unlockedTime = ScoringSDK.getUnlockedTime() || 0;
            if (difference > unlockedTime && !self._unlock && self._window) {
                self._window.show();
            }
        },
        _loadParameters: function () {
            var self = this,
                time = ScoringSDK.load((self.memoryKey + "time")) || "";
            self._time = time ? new Date(time) : new Date();
            self._unlock = ScoringSDK.load((self.memoryKey + "unlock")) || false;
            self._innap = ScoringSDK.load((self.memoryKey + "innap")) || false;
            self._checking = ScoringSDK.load((self.memoryKey + "checking")) || false;
        },
        _saveParameters: function () {
            var self = this;
            ScoringSDK.save((self.memoryKey + "time"), self._time);
            ScoringSDK.save((self.memoryKey + "unlock"), self._unlock);
            ScoringSDK.save((self.memoryKey + "innap"), self._innap);
            ScoringSDK.save((self.memoryKey + "checking"), self._checking);
        },
        _setTimerForCheckingTime: function () {
            var self = this;
            if (self._timer.checkTime) return;

            self._timer.checkTime = setInterval(function () {
                self.checkTime();
            }, self.intervalChecking.time);
        },
        _setTimerForCheckingInnap: function () {
            var self = this;
            if (self._timer.checkInnap) return;

            self._timer.checkInnap = setInterval(function () {
                self.checkInnap();
            }, self.intervalChecking.innap);
        },
        _setWindow: function () {
            this._window = module.popupWindow("popup_window__innap");
        },
        _setNodes: function () {
            var arrayID = [
                "button",
                "phoneNumber",
                "textSMS",
                "loading"
            ];
            for (var i = 0, l = arrayID.length; i < l; i++) {
                this._nodes[(arrayID[i])] = document.getElementById((this.prefix + arrayID[i]));
            }
        },
        _showLoading: function () {
            var node = this._nodes.loading;
            if (node) node.className = "innap__check_container";
        },
        _setInnap: function () {
            var self = this;
            ScoringSDK.getInnap(function (innap) {
                self._innap = innap;
                self._render();
                if (!innap) return;
                self._saveParameters();
                self._setEventForButton();
            });
        },
        checkInnap: function () {
            var self = this;
            ScoringSDK.checkInnap(self._innap, function (status) {
                if (!status) return;
                self._unlock = true;
                self._time = new Date();
                self._clearTimer();
                self._saveParameters();
                self._window.hide();
            });
        },
        _render: function () {
            var self = this,
                nodes = self._nodes,
                innap = self._innap || {};
            if (nodes.phoneNumber) nodes.phoneNumber.innerHTML = innap.phone || "";
            if (nodes.textSMS) nodes.textSMS.innerHTML = innap.body || "";
        },
        _setEventForButton: function () {
            var self = this,
                node = self._nodes.button;
            utils.addEvent(node, "click", function () {
                self._checking = true;
                self._saveParameters();
                self.checkTime();
                self._setTimerForCheckingInnap();
                self._showLoading();
                self.checkInnap();
                window.location.href = "sms:" + (self._innap.phone || "") + "?body=" + (self._innap.body || "");
            });
        },
        _clearTimer: function () {
            var timer = this._timer;
            clearInterval(timer.checkTime);
            clearInterval(timer.checkInnap);
        },
        initialization: function () {
            var self = this;
            self._loadParameters();
            if (self._unlock) return false;

            self._setTimerForCheckingTime();
            self._setWindow();
            self._setNodes();

            if (!self._innap) {
                self._setInnap();
                return;
            }

            self._render();
            self._saveParameters();
            self._setEventForButton();
            self._setTimerForCheckingInnap();

            if (!self._checking) return;
            self._showLoading();
            self.checkInnap();
            self.checkTime();
        }
    };

    module.object.set(Innap, methods);

    module.api.create(function (data) {
        var object = new Innap(data);
        object.initialization();
        return object;
    }, "innap");
})(this);