/*
 * Модуль языков.
 * Ищет DOM элементы с data-language = "ID_перевода"
 * Если находит делает замену.
 *
 * - Почему не выбран стандартный i18n?
 * Потому что в большинстве реализаций:
 *  - он работает через jQuery
 *  - не работает offline, из-за AJAX запросов
 *  - нет функционала по принудительной смене языка
 *  - требует изменения классов элементов (это нарушение зоны ответственности модуля)
 *
 * Минусы этого модуля:
 * - Формат языков не JSON в чистом виде
 * - При отказе модуля, есть гломальные зависимости, которые могут привести к отказу всей системы
 *
 * Слушает события:
 * language__update - обновить DOM элементы на странице.
 * language__change - сменить язык. Ожидает ID языка.
 *
 * Модуль создает глобальный объект language
 * API модуля:
 *   get - получить перевод. Ожидает ID фразы.
 *   set - установить переводы. Ожидает объект с фразами.
 *
 * Настройки модуля:
 *  _language          - язык по умолчанию
 *  _path              - путь к файлам переводов
 *  _correctLanguages  - допустимые языки
 *
 * Пример вызова:
 * language.get("save");
 *
 * Добавит:
 * <span data-language="save">Сохранить</span>
 *
 * При обновление языка фраза будет найдена и изменена.
 *
 */
(function (global) {
    "use strict";

    global = global || {};

    var module = {
        _language: "ru",
        _path: "module/language/translations/",
        _correctLanguages: {
            ru: true,
            en: true
        },
        _data: {},
        _setEvents: function () {
            var self = this;
            utils.event.listen("language__update", function () {
                self.updateDOM(document.body);
            });
            utils.event.listen("language__change", function (id) {
                self._setLanguage(id);
                self._setScript();
            });
        },
        _setLanguage: function (id) {
            var self = this,
                correct = self._correctLanguages;
            if (global.navigator && global.navigator.language) {
                var data = global.navigator.language.toLowerCase().match(/^[a-z]{2}/gim);
                if (data && data[0] && correct[(data[0])]) self._language = data[0];
            }
            if (id && correct[id]) self._language = id;
        },
        _setScript: function () {
            var self = this,
                parent = document.head,
                node = document.createElement("SCRIPT");
            node.setAttribute("src", (self._path + self._language + ".js"));
            if (!parent) return false;
            parent.appendChild(node);
            utils.addEvent(node, "load", function () {
                self.updateDOM(document.body);
                parent.removeChild(node);
            });
            utils.addEvent(node, "error", function () {
                parent.removeChild(node);
            });
        },
        updateDOM: function (parentNode) {
            if (!parentNode || !parentNode.childNodes) return false;
            var self = this;
            for (var i = 0, l = parentNode.childNodes.length; i < l; i++) {
                var item = parentNode.childNodes[i];
                if (item.nodeType != 1) continue;
                var id = item.getAttribute("data-language");
                if (id && self._data[id]) {
                    item.innerHTML = self._data[id];
                } else {
                    self.updateDOM(item);
                }
            }
        },
        set: function (data) {
            if (!data || typeof data != "object") return false;
            for (var id in data) this._data[id] = data[id] || "";
        },
        get: function (id) {
            if (!id || typeof id != "string") return "";
            return ("<span data-language=\"" + id + "\">" + (this._data[id] || "") + "</span>");
        },
        clear: function () {
            global.language = {};
        },
        init: function () {
            var self = this;
            self._setLanguage();
            self._setScript();
            self._setEvents();
        }
    };

    module.init();

    global.language = {
        get: function (id) {
            return module.get(id);
        },
        set: function (data) {
            module.set(data);
        }
    };
})(this);