﻿/*
 * Модуль для работы с объектами.
 * Модуль можно использовать при создании нового объекта и при задание параметров по умолчанию.
 * Например:
 * var defaultConfig = {
 *         styles: [ "default" ],
 *         package: 0,
 *         templates: {
 *             teaser: "test_result__vk"
 *         }
 *     };
 * module.object.set(this, defaultConfig, memoryConfig, userConfig);
 * 
 * где:
 *   this - объект, которому будут унаследованны свойства.
 *   defaultConfig - параметры по умолчанию.
 *   memoryConfig  - параметры из памяти.
 *   userConfig    - параметры введенные пользователем.
 * 
 * API модуля:
 * copy() - копировать объект. Ожидает один аргумент - объект, который необходимо копировать.
 * 
 * get()  - создать новый объект и наследовать ему свойства других объектов.
 *          Ожидает набор объектов, из которых необходимо создать новый. Аналог функции mixin().
 * 
 * set()  - наследовать одному объекту, свойства других объектов.
 *          Ожидает исходный объект и набор объектов-доноров, из которых необходимо наследовать свойства. 
 *          Аналог функции mixin().
 * 
 * toObject() - преобразовать массив объектов в объект с методами. Ожидает два аргумента: массив и название свойства,
 *              которое преобразуется в название метода. Например:
 *              [                       {
 *                  { id: "a" },   ->      a: { id: "a" },
 *                  { id: "a" },           b: { id: "b" }
 *              ]                       }
 * 
 * toArray() - преобразовать объект с методами в массив объектов. Ожидает один аргумент: объект. Например:
 *             {                        [ 
 *                 a: { id: "a" }, ->      { id: "a" },
 *                 b: { id: "b" }          { id: "a" },
 *             }                        ]
 * 
 * getProperty() - получает вложенное свойство объекта по строке адреса. Ожидает два аргумента: объект и название свойства.
 *                 Например:
 *                 var x = { a: { b: { c: "key" } } },
 *                     y = "a.b.c";
 *                 getProperty(x, y) вернет "key"
 * 
 * getObjectFromString() - обертка над JSON.parse с try/catch конструкцией.
 * 
 */
(function (global) {
    "use strict";

    global = global || {};
    global.StalinGrad = global.StalinGrad || {};
    var utils = global.StalinGrad.utils || {},
        module = global.StalinGrad.module || {};

    var methods = {
        _inherit: function (object, property) {
            var self = this;
            if (!object || !property) return false;
            for (var id in property) {
                if (property[id] && typeof property[id] == "object") {
                    if (!object[id] || typeof object[id] != "object") object[id] = (property[id] instanceof Array) ? [] : {};
                    self._inherit(object[id], property[id]);
                } else object[id] = property[id];
            }
        },
        _inheritList: function (object, data, index) {
            for (var i = index || 0, l = data.length; i < l; i++) {
                if (typeof data[i] == "string") data[i] = this.getObjectFromString(data[i]);
                if (typeof data[i] == "object" && data[i]) this._inherit(object, data[i]);
            }
            return object;
        },
        getObjectFromString: function (content) {
            try {
                if (!global.JSON || !global.JSON.parse) return {};
                return JSON.parse(content) || {};
            } catch (e) {
                return {};
            }
        },
        _inheritPrototypes: function (object, arrayListsPrototypes) {
            for (var i = 1, l = arrayListsPrototypes.length; i < l; i++) {
                if (typeof arrayListsPrototypes[i] != "object") continue;
                for (var nameMethod in arrayListsPrototypes[i]) {
                    object.prototype[nameMethod] = arrayListsPrototypes[i][nameMethod];
                }
            }
        },
        toObject: function (data, key) {
            var object = {};
            for (var i = 0, l = data.length; i < l; i++) {
                object[(key || i)] = data[i];
            }
            return object;
        },
        toArray: function (object) {
            var data = [];
            for (var id in object) {
                data.push(object[id]);
            }
            return data;
        },
        getProperty: function (object, propertyName) {
            if (typeof propertyName != "string") return null;
            if (typeof object != "object") return null;

            propertyName = propertyName.split(/[\.]+/gim);
            for (var i = 0, l = propertyName.length; i < l; i++) {
                if (!object[(propertyName[i])]) return null;
                object = object[(propertyName[i])];
            }
            return object;
        },
        copy: function (data) {
            if (data instanceof Array) {
                var newArray = [];
                for (var i = 0, l = data.length; i < l; i++) newArray[i] = data[i];
            }
            return newArray;
        },
        set: function () {
            if (typeof arguments[0] == "function") {
                this._inheritPrototypes(arguments[0], arguments);
            } else {
                this._inheritList((arguments[0] || {}), arguments, 1);
            }
        },
        get: function () {
            return this._inheritList({}, arguments);
        }
    };

    module.object = {
        toArray: function (object) {
            return methods.toArray(object);
        },
        toObject: function (data, key) {
            return methods.toObject(data, key);
        },
        getProperty: function () {
            return methods.getProperty.apply(methods, arguments);
        },
        getObjectFromString: function () {
            return methods.getObjectFromString(object);
        },
        copy: function (object) {
            return methods.copy(object);
        },
        get: function () {
            return methods.get.apply(methods, arguments);
        },
        set: function () {
            methods.set.apply(methods, arguments);
        }
    };
})(this);