/*
 * Окна.
 * Нужен, когда накопилось достаточное количество различных окон, которые нужно отображать пользователю в ответ на различные события (например, инвентарь, задания, рейтинг, схема прокачки и т.д.)
 * Чтобы упорядочить эти окна и получить удобный интерфейс управления ими - используйте данный модуль.
 *
 * Пример использования:
 * var win1 = core.module.popupWindow("window_1");    // При создание нового окна, требуется ID html элемента
 * var win2 = core.module.popupWindow("window_2");
 * var win3 = core.module.popupWindow("window_3");
 *
 * Каждое окно обладает следующим API:
 * show() - покажет окно
 * hide() - скроет окно
 *
 * Если было открыто несколько окон, например, так:
 * win1.show();
 * win2.show();
 * То при открытии - каждое новое окно скрывает прошлое, при закрытие - каждое закрытое, восстанавливает прошлое окно.
 * То есть в модуле имеется стек, который помнит, в каком порядке открывали окна.
 *
 */

(function (global) {
    "use strict";

    global = global || {};
    global.StalinGrad = global.StalinGrad || {};
    var utils = global.StalinGrad.utils || {},
        module = global.StalinGrad.module || {};

    function PopupWindow(data) {
        var self = this;
        self.id = data.id;
        self.node = data.node;
        self.isShow = false;
    }

    PopupWindow.prototype.show = function () {
        this.isShow = true;
        methods.show({
            id: this.id,
            node: this.node
        });
    };

    PopupWindow.prototype.hide = function () {
        this.isShow = false;
        methods.hide({
            id: this.id,
            node: this.node
        });
        methods.closeTable();
    };

    PopupWindow.prototype.toggle = function () {
        var type = (this.isShow) ? "hide" : "show";
        methods[type]({
            id: this.id,
            node: this.node
        });
    };

    var methods = {
        isShow: false,
        _stack: [],
        _nodes: {},
        _prefix: "popup_window__",
        _setNode: function () {
            var arrayID = [
                "table",
                "cell"
            ];
            for (var i = 0, l = arrayID.length; i < l; i++) {
                this._nodes[(arrayID[i])] = document.getElementById((this._prefix + arrayID[i]));
            }
        },
        _setEvent: function () {
            var self = this;
            utils.addEvent(self._nodes.table, "click", function (event) {
                utils.stopEvent(event);
                self.closeTable();
            });
        },
        closeTable: function () {
            var self = this,
                node = self._nodes.table;
            if (node) {
                node.className = "popup_window__table_close";
                setTimeout(function () {
                    node.className = "popup_window__hidden";
                    self._closeAllWindow();
                    self.isShow = false;
                }, 480);
            }
            utils.event.trigger("popup_window__hidden");
        },
        _closeAllWindow: function () {
            var self = this;
            for (var i = 0, l = self._stack.length; i < l; i++) {
                var item = self._stack[i];
                if (item.node) item.node.className = "popup_window__hidden";
            }
            self._stack = [];
        },
        close: function (data) {
            var self = this;
            for (var i = 0, l = self._stack.length; i <= l; i++) {
                var item = self._stack[i];
                if (item.id != data.id) continue;
                if (item.node) item.node.className = "popup_window__hidden";
                self._stack.splice(i, 1);
                break;
            }
            if (!self.isShow) return false;
            if (self._stack.length === 0) {
                self.closeTable();
            } else {
                self.show(self._stack[(self._stack.length - 1)]);
            }
        },
        hide: function (data) {
            if (data.node) data.node.className = "popup_window__hidden";
        },
        show: function (data) {
            var self = this,
                nodeTable = self._nodes.table;
            if (!data) return false;

            var lastIndex = self._stack.length - 1;

            if (lastIndex >= 0) {
                if (self._stack[lastIndex].id != data.id) {
                    self._stack.push(data);
                    self.hide(self._stack[lastIndex]);
                }
            } else {
                self._stack.push(data);
            }

            if (data.node) {
                data.node.className = "popup_window__item";
                data.node.focus();
            }

            if (!self.isShow && nodeTable) {
                nodeTable.className = "popup_window__table";
            }

            self.isShow = true;
        },
        _createCloseButton: function (parentNode) {
            if (!parentNode) return false;
            var node = document.createElement("IMG");
            node.className = "popup_window__item_close";
            node.setAttribute("src", "module/popup_window/popup_window.png");
            parentNode.appendChild(node);
            return node;
        },
        _addEventForWindow: function (nodeWindow, nodeButton, data) {
            var self = this;
            utils.addEvent(nodeButton, "click", function (event) {
                utils.stopEvent(event);
                self.close(data);
            });
            utils.addEvent(nodeWindow, "click", function (event) {
                utils.stopEvent(event);
            });
        },
        createWindow: function (id) {
            var self = this,
                nodeWindow = document.getElementById(id),
                nodeButton = self._createCloseButton(nodeWindow),
                data = {
                    id: id,
                    node: nodeWindow
                };
            self._addEventForWindow(nodeWindow, nodeButton, data);
            return new PopupWindow(data);
        },
        initialization: function () {
            var self = this;
            if (self._nodes.table) return false;
            self._setNode();
            self._setEvent();
        }
    };

    module.api.create(function (id) {
        return methods.createWindow(id);
    }, "popupWindow");

    utils.ready(function () {
        methods.initialization();
    });
})(this);