/*
 * Таймер.
 *
 * Добавляет таймер в верхний правый угол. Может быть полезен в играх для отсчета времени.
 *
 * Структура модуля:
 *   _nodes  <- объект, содержит ссылки на DOM элементы
 *   _time   <- счетчик секунд
 *   _timer  <- ссылка на экземпляр таймера
 *   _prefix <- префикс для классов и ID элементов DOM дерева
 *   _resize <- функция, задает размер таймера относительно размера экрана
 *   _setEvents <- функция, устанавливает обработчики событий
 *   _render    <- функция, отвечает за отображение времени на дисплее
 *   _start     <- функция, запускает таймер
 *   _setNodes  <- функция, устанавливает ссылки на DOM элементы таймера
 *   _show <- функция, показывает таймер после инициализации
 *   init  <- функция инициализации
 *
 * Слушает события:
 * timer_up__start    - запустить таймер
 * timer_up__pause    - приостановить таймер
 * timer_up__clear    - обнулить и запустить таймер
 * timer_up__resize   - обновить размер таймера
 * timer_up__set_time - установить время на таймере
 *
 * Публикует события:
 * timer_up__tick     - тик таймера, событие публикуется каждую секунду
 *
 */
(function (global) {
    "use strict";

    global = global || {};
    var utils = global.StalinGrad.utils;

    var module = {
        _nodes: {},
        _time: 0,
        _timer: null,
        _prefix: "timer_up__",
        _resize: function () {
            var node = this._nodes,
                w = utils.display.width(),
                h = utils.display.height(),
                min = Math.ceil(Math.min(w, h) * 0.07);
            if (!node.container) return false;
            if (min < 30) min = 30;
            if (min > 40) min = 40;
            node.container.style.fontSize = min + "px";
        },
        _setEvents: function () {
            var self = this;
            utils.event.listen("timer_up__start", function () {
                self._start();
            });
            utils.event.listen("timer_up__pause", function () {
                clearInterval(self._timer);
            });
            utils.event.listen("timer_up__clear", function () {
                self._time = 0;
                self._start();
            });
            utils.event.listen("timer_up__resize", function () {
                self._resize();
            });
            utils.event.listen("timer_up__set_time", function (time) {
                time = parseInt(time, 10);
                self._time = time || 0;
            });
            utils.event.listen("timer_up__hide", function (time) {
                self._hide();
            });
        },
        _render: function () {
            var self = this,
                node = self._nodes,
                hours = Math.floor(self._time / 3600),
                minutes = Math.floor(self._time / 60) - hours * 60,
                seconds = self._time - hours * 3600 - minutes * 60;
            if (hours > 23) hours = 23;
            if (hours < 10) hours = "0" + hours;
            if (minutes < 10) minutes = "0" + minutes;
            if (seconds < 10) seconds = "0" + seconds;
            if (node.minutes) node.minutes.innerHTML = minutes || "0";
            if (node.seconds) node.seconds.innerHTML = seconds || "0";
            if (node.hours) node.hours.innerHTML = hours || "0";
        },
        _start: function () {
            var self = this;
            clearInterval(self._timer);
            self._timer = setInterval(function () {
                self._time++;
                self._render();
                utils.event.trigger("timer_up__tick", self._time);
            }, 1000);
        },
        _setNodes: function () {
            var arrayID = [
                "container",
                "hours",
                "minutes",
                "seconds"
            ];
            for (var i = 0, l = arrayID.length; i < l; i++) {
                this._nodes[(arrayID[i])] = document.getElementById((this._prefix + arrayID[i]));
            }
        },
        _show: function () {
            var node = this._nodes.container;
            if (node) node.className = "timer_up__container";
        },
        _hide: function () {
            var node = this._nodes.container;
            if (node) node.className = "hidden";
        },
        init: function () {
            var self = this;
            self._setNodes();
            self._setEvents();
            self._start();
            self._show();
            self._resize();
        }
    };

    utils.ready(function () {
        module.init();
    });
})(this);